<?php

namespace Drupal\Tests\trailless_menu\Functional;

use Drupal\menu_link_content\Entity\MenuLinkContent;
use Drupal\Tests\BrowserTestBase;

/**
 * Tests that trailless menu doesn't expand and tests that render happens only
 * once for trailless menu.
 *
 * @group trailless_menu
 */
class TraillessMenuTest extends BrowserTestBase {

  /**
   * An editor user.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $editor;

  /**
   * An authenticated user to test block caching.
   *
   * @var object
   */
  protected $normalUser;

  /**
   * Modules to enable.
   *
   * @var array
   */
  protected static $modules = [
    'system',
    'block',
    'menu_link_content',
    'node',
    'trailless_menu',
    'trailless_menu_render_count_test',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  protected function setUp(): void {
    parent::setUp();

    $this->drupalCreateContentType([
      'type' => 'page',
      'name' => 'Basic page',
      'display_submitted' => FALSE,
    ]);

    $this->block = $this->drupalPlaceBlock(
      'system_menu_block:main',
      [
        'label' => 'trailless_menu main navigation'
      ]
    );

    $this->editor = $this->drupalCreateUser([
      'access administration pages',
      'administer blocks',
      'administer menu',
      'create page content',
      'administer trailless menu',
    ]);
    $this->drupalLogin($this->editor);

    // Create additional users to test caching modes.
    $this->normalUser = $this->drupalCreateUser();

    // Create nodes.
    $first = $this->drupalCreateNode();
    $second = $this->drupalCreateNode();

    // Create menu links for nodes.
    $this->first_item = MenuLinkContent::create([
      'link' => [['uri' => 'entity:node/' . $first->id()]],
      'title' => 'first',
      'menu_name' => 'main',
    ]);
    $this->first_item->save();

    $this->second_item = MenuLinkContent::create([
      'link' => [['uri' => 'entity:node/' . $second->id()]],
      'title' => 'second',
      'menu_name' => 'main',
      'parent' => 'menu_link_content:' . $this->first_item->uuid(),
    ]);
    $this->second_item->save();

    // Goto trailless_menu setting page and tick main menu and save form.
    $edit = [
      'trailless_menus[main]' => TRUE,
    ];
    $this->drupalGet('/admin/config/user-interface/trailless-menu');
    $this->submitForm($edit, 'Save configuration');
  }

  /**
   * Tests that trailless menu doesn't expand.
   */
  public function testTraillessMenuLink() {
    $this->drupalLogin($this->normalUser);

    // Goto a parent menu link and verify that parent link exists on the page.
    $this->drupalGet($this->first_item->getUrlObject());
    $this->assertSession()
      ->linkByHrefExists($this->first_item->getUrlObject()->toString());

    // Goto its child and verify that child link doesn't exists on the page.
    $this->drupalGet($this->second_item->getUrlObject());
    $this->assertSession()
      ->linkByHrefNotExists($this->second_item->getUrlObject()->toString());
  }

  /**
   * Tests that render happens only once for trailless menu.
   */
  public function testRenderHappensOnceForTraillessMenu() {
    $this->drupalLogin($this->normalUser);
    $state = \Drupal::state();

    // Resetting state cache so that we don't get cache state value.
    $state->resetCache();

    // Getting render counter, which is set from trailless_menu_render_count_test
    // test module. Its value is increased everytime system_menu_block:main
    // block is rendered.
    $render_counter = $state->get('render_counter', 0);

    // Invalidating cache tags so that it freshly renders the menu block.
    \Drupal::service('cache_tags.invalidator')
      ->invalidateTags(['config:system.menu.main']);

    // Goto a parent menu link and verify that it is not re-rendered.
    $this->drupalGet($this->first_item->getUrlObject());

    // Resetting state cache so that we don't get cache state value.
    $state->resetCache();

    $this->assertSame(++$render_counter, $state->get('render_counter'));

    // Goto its child and verify that it is not re-rendered.
    $this->drupalGet($this->second_item->getUrlObject());

    // Resetting state cache so that we don't get cache state value.
    $state->resetCache();

    $this->assertSame($render_counter, $state->get('render_counter'));
  }

  /**
   * Tests that cache gets invalidated upon trailless_menu form save if form
   * setting is changed.
   */
  function testCacheInvalidationOnFormSave() {
    /** @var \Drupal\Core\Cache\DatabaseCacheTagsChecksum $cacheTagChecksum **/
    $cacheTagChecksum = \Drupal::service('cache_tags.invalidator.checksum');

    // Create menu links for footer menu.
    $this->first_item = MenuLinkContent::create([
      'link' => 'https://example.com',
      'title' => 'footer1',
      'menu_name' => 'footer',
    ]);
    $this->first_item->save();

    $this->second_item = MenuLinkContent::create([
      'link' => 'https://example.com/footer2',
      'title' => 'footer2',
      'menu_name' => 'footer',
    ]);
    $this->second_item->save();

    // Place footer menu on the page.
    $this->block = $this->drupalPlaceBlock(
      'system_menu_block:footer',
      [
        'label' => 'trailless_menu footer navigation'
      ]
    );

    // Goto home page or some other page.
    $this->drupalGet('');

    // Goto trailless menu config page.
    $this->drupalGet('/admin/config/user-interface/trailless-menu');

    // Check cachetag count for tag reset of main and footer.
    $mainMenuCount = $cacheTagChecksum
      ->getCurrentChecksum(['config:system.menu.main']);
    $footerMenuCount = $cacheTagChecksum
      ->getCurrentChecksum(['config:system.menu.footer']);

    // Tick footer in the config page and save config.
    $edit = [
      'trailless_menus[main]' => TRUE,
      'trailless_menus[footer]' => TRUE,
    ];
    $this->submitForm($edit, 'Save configuration');

    // Check that main menu cachetag count is same and for footer is changed.
    $newMainMenuCount = $cacheTagChecksum
      ->getCurrentChecksum(['config:system.menu.main']);
    $newFooterMenuCount = $cacheTagChecksum
      ->getCurrentChecksum(['config:system.menu.footer']);

    $this->assertSame($mainMenuCount, $newMainMenuCount);
    $this->assertNotSame($footerMenuCount, $newFooterMenuCount);

    // Untick main menu and save config.
    $this->drupalGet('/admin/config/user-interface/trailless-menu');
    $edit['trailless_menus[main]'] = FALSE;
    $this->submitForm($edit, 'Save configuration');

    // Check that footer menu cachetag count is same and for main menu is changed.
    $mainMenuCount = $newMainMenuCount;
    $footerMenuCount = $newFooterMenuCount;

    $newMainMenuCount = $cacheTagChecksum
      ->getCurrentChecksum(['config:system.menu.main']);
    $newFooterMenuCount = $cacheTagChecksum
      ->getCurrentChecksum(['config:system.menu.footer']);

    $this->assertNotSame($mainMenuCount, $newMainMenuCount);
    $this->assertSame($footerMenuCount, $newFooterMenuCount);
  }
}
